{{
***************************************************
* File jvmConst: pushing constants onto the stack *
***************************************************
}}


OBJ
  js: "jvmStack"
  jc: "jvmConvert"
  jd: "jvmData"
  
  
PUB jb_bipush | val
{{ Meaning: Push a 1-byte signed integer. }}
{{
  JEM format: bipush,byte1
  byte1 is a signed 1byte constant: if byte1 < 128 const = byte1 else const = byte1 | $FF00 
}}
  val := jd.readJavaByte(jd.getJvmPC+1)
  val := jc.jb_b2i(val)
  js.jb_push(val)
  jd.incJvmPC(2)
  
  
PUB jb_sipush | val
{{ Meaning: Push a 2-byte signed integer. }}
{{
  JEM format: sipush,byte1,byte2
  byte1 and byte2 form 16bits constant: const = (byte1<<8)|(byte2) 
}}
  val := jd.readJavaIntBE(jd.getJvmPC+1)
  js.jb_push(val)
  jd.incJvmPC(3)
  

PUB jb_ldc | index,val
{{ Meaning: Push item from constant pool. }}
{{
  JEM format: ldc,byte1
  byte1 is index in String Indextable
}}
  index := jd.readJavaByte(jd.getJvmPC+1)
  val := jd.getLdc(index)
  js.jb_push(val) 
  jd.incJvmPC(2)
  
   
PUB jb_ldc_w | index,val
{{Meaning: Push 2-byte item from constant pool. }}
{{
  JEM format: ldc_w,byte1,byte2
  byte1 is always 0. byte2 is index in String Indextable
}}
  index := jd.readJavaByte(jd.getJvmPC+2)
  val := jd.getLdc(index)
  js.jb_push(val) 
  jd.incJvmPC(3)


{{PUB jb_ldc_w2}}
{{ Meaning: Push long or double from constant pool. }}


PUB jb_aconst_null
{{ Meaning: Push a null object reference. }}
{{
  JEM format: aconst_null
}}
  js.jb_push($0000)
  jd.incJvmPC(1)
  
  
PUB jb_iconst_m1
{{ Meaning: Push the integer constant -1. }}
{{
  JEM format: iconst_m1
}}
  js.jb_push($FFFF)
  jd.incJvmPC(1)

  
PUB jb_iconst_0
{{ Meaning: Push the integer constant 0. }}
{{
  JEM format: iconst_0
}}
  js.jb_push($0000)
  jd.incJvmPC(1)
  

PUB jb_iconst_1
{{ Meaning: Push the integer constant 1. }}
{{
  JEM format: iconst_1
}}
  js.jb_push($0001)
  jd.incJvmPC(1)


PUB jb_iconst_2
{{ Meaning: Push the integer constant 2. }}
{{
  JEM format: iconst_2
}}
  js.jb_push($0002)
  jd.incJvmPC(1)


PUB jb_iconst_3
{{ Meaning: Push the integer constant 3. }}
{{
  JEM format: iconst_3
}}
  js.jb_push($0003)
  jd.incJvmPC(1)


PUB jb_iconst_4
{{ Meaning: Push the integer constant 4. }}
{{
  JEM format: iconst_4
}}
  js.jb_push($0004)
  jd.incJvmPC(1)


PUB jb_iconst_5
{{ Meaning: Push the integer constant 5. }}
{{
  JEM format: iconst_5
}}
  js.jb_push($0005)
  jd.incJvmPC(1)


{{PUB jb_lconst_0}}
{{ Meaning: Push the long integer constant #0. }}


{{PUB jb_lconst_1}}
{{ Meaning: Push the long integer constant #1. }}


{{PUB jb_fconst_0}}
{{ Meaning: Push the single-precision floating-point number #0. }}


{{PUB jb_fconst_1}}
{{ Meaning: Push the single-precision floating-point number #1. }}


{{PUB jb_fconst_2}}
{{ Meaning: Push the single-precision floating-point number #2. }}


{{PUB jb_dconst_0}}
{{ Meaning: Push the double-precision floating-point number #0. }}


{{PUB jb_dconst_1}}
{{ Meaning: Push the double-precision floating-point number #1. }}

