{{
**********************************************************
* File jvmInvoke: invoking methods and exception handler *
**********************************************************
}}

{{
  Invoking Methods. 
  Method invocation is a complex process. To invoke a method, the operand stack must contain a reference 
  to an object and some number of arguments. The object reference is used as a pointer to the object's 
  method table, which contains the method signature. The method signature is guaranteed to exactly match 
  one of the method signatures in the table. The arguments byte1 and byte2 are used to construct an index 
  into the constant pool of the current class, which contains the complete method signature.  
  The result of the lookup is an index into the method table of the named class. This index is used with the 
  referenced object's dynamic type to look in the method table of that type, where a pointer to the method 
  block for the matched method is found. The method block indicates the type of method, such as native or 
  synchronized, and the number of arguments expected on the operand stack.  
  The object reference and arguments are popped off the method's operand stack and become the initial 
  values of the local variables of the new method. Execution continues with the first instruction of the new 
  method. 
  Note:
  The monitor associated with the referenced object is entered if the method is marked as synchronized.
  A NullPointerException is thrown if the object reference on the operand stack is null. A StackOverflowError
  is thrown if a stack overflow is detected during the method invocation.

  Exception Handling.  
  Because exception handling is a major feature of the Java programming language, you might be surprised 
  to learn that exceptions are handled with a single bytecode instruction. When exceptions occur, the object 
  thrown must be a reference to an object of the subclass Throwable, and the current Java stackframe is 
  searched for the most recent catch clause that catches exceptions of this class or a superclass of this 
  class. When a matching catch clause is found, the program counter is reset to the address indicated by 
  the catch clause and execution continues from there. When no appropriate catch clause is found, that 
  frame is popped and the object is rethrown. If a catch clause is then found, the clause will contain the 
  location of the code for the exception, the program counter is reset to that location, and execution 
  continues. Otherwise, the frame is popped and the object is rethrown. 
  Note:
  A NullPointerException is thrown instead if the referenced object is null.  
}}


OBJ
  js: "jvmStack"
  jd: "jvmData"
  
  
PUB jb_invokevirtual
{{ Meaning: Invoke method based on runtime type. }}
{{
  JEM format: invokevirtual,byte1,byte2
  byte2 is offset to begin of class method table (so if byte = $2D and class method table = $60, method header = $8D)
  byte1 is offset to sp to find object reference (so if sp = $3FF6 and byte1 = $04 then the aref is at $3FF6-$04 = $3FF2)
}}


{{PUB jb_invokenonvirtual}}
{{ Meaning: Invoke method based on compile-time type. }}


PUB jb_invokestatic
{{ Meaning: Invoke a class (static) method. }}
{{
  JEM format: invokestatic,byte1,byte2
  byte1 and byte2 form 16bits address to method header: addr = (byte1<<8)|(byte2) 
}}


{{PUB jb_invokeinterface}}
{{ Meaning: Invoke interface method. }} 
{{
  Note:
  Unlike invokevirtual and invokenonvirtual, the method block does not indicate the number
  of available arguments. This number is taken from the bytecode.
}}


PUB jb_athrow
{{ Meaning: Throw exception or error. }}
{{
  JEM format: athrow
}}

